/*
 *  diskdumpfmt.c
 *
 *  Copyright (C) 2004  FUJITSU LIMITED
 *  Written by Akira Imamura (imamura.akira@jp.fujitsu.com)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
/* $Id: diskdumpfmt.c,v 1.77 2004/10/13 19:19:24 tachino Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "diskdump_format.h"

#define Dbg(x...) do { if (verbose) fprintf(stderr, x); } while (0);
#define Err(x...) fprintf(stderr, "diskdumpfmt: " x)


static void usage(void) {
	fprintf(stderr, "Usage: diskdumpfmt [-v] [[-p] [-f]|-c] device\n");
	exit(255);
}

int main(int argc, char *argv[])
{
	dump_device_t *device;
	int opt;
	int chk_flg = 0, frc_flg = 0, prg_flg = 0, verbose = 0;
	int ret;
	char *dev_name;

	while ((opt = getopt(argc, argv, "cfpv")) != -1) {
		switch (opt) {
		case 'c' :
			chk_flg = 1;
			break;
		case 'f' :
			frc_flg = 1;
			break;
		case 'p' :
			prg_flg = 1;
			break;
		case 'v' :
			verbose = 1;
			break;
		default:
			usage();
			break;
		}
	}

	if ((chk_flg && frc_flg) || (chk_flg && prg_flg))
		usage();

	if (optind + 1 != argc)
		usage();

	dev_name = argv[optind];
	Dbg("device name  : %s\n", dev_name);

	device = open_device(dev_name);
	if (device == NULL) {
		Err("cannot open target device (%s), %s\n", dev_name, strerror(errno));
		exit(255);
	}

	set_progress(device, prg_flg);

	if (chk_flg) {
		if ((ret = check_diskdump_signature(device, 0)) == -1) {
			Dbg("for diskdump?: no\n");
			Dbg("any dump?    : n/a\n");
			exit(2);
		} else if ((ret = check_diskdump_signature(device, 1)) == -1) {
			Dbg("for diskdump?: yes\n");
			Dbg("any dump?    : yes\n");
			exit(1);
		} else if (ret == 0) {
			Dbg("for diskdump?: yes\n");
			Dbg("any dump?    : no\n");
			exit(0);
		} else {
			Err("cannot read target device correctly (%s)\n", dev_name);
			exit(255);
		}
	}

	if (!frc_flg) {
		ret = check_diskdump_signature(device, 0);
		if (ret == -1) {
			Dbg("for diskdump?: no\n");
			Err("initial format required\n");
			exit(2);
		} else if (ret == 255) {
			Err("cannot read target device correctly (%s)\n", dev_name);
			exit(255);
		}
	}

	if (write_diskdump_signature(device) == -1) {
		Err("cannot write diskdump signature.\n");
		exit(255);
	}
	Dbg("block size   : %Lu\n", (unsigned long long)get_block_size(device));
	Dbg("total size   : %Lu\n", (unsigned long long)get_block_size(device) * get_page_size(device));

	Dbg("diskdumpfmt  : formatting has been done.\n");

	exit(0);
}

